# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        scrolled_list.py
# Description: Transparent Scrolled List Widget
# Copyright (c) 2006   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import sys

#gtk
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import pango

class ScrollList:
    """custom scroll list control"""
    def __init__(self, parent=None, num_rows=10, changed_cb=None):
        """create custom scroll list control"""
        self.parent = parent
        self.width, self.height = 100, 200
        self.num_rows = num_rows
        self.ls = []
        self.display_limiters = False
        self._row_height = self.height / self.num_rows
        self._hl_on_row = 0
        self._ls_idx = 0
        #default colours
        self._hl_bg_col = gtk.gdk.color_parse('white')
        self._hl_fg_col = gtk.gdk.color_parse('green')
        self._fg_col = gtk.gdk.color_parse('yellow')
        #create the list
        self._create_container()
        self._create_list_labels()
        #size
        self.set_size_request(self.width, self.height)
        #set callbacks
        self.changed_cb = changed_cb

    def set_selected(self, ls_index):
        """scroll to and highight given item in list"""
        if len(self.ls) == 0:
            self._update_display()
            return
        if ls_index > len(self.ls) - 1:
            ls_index  = len(self.ls) - 1
            #print "ls_index=", ls_index
            #print "len(self.ls)=", len(self.ls)
            #raise IndexError
        #remove highlight
        self.rows[self._hl_on_row][0].set_visible_window(False)
        self.rows[self._hl_on_row][1].modify_fg(gtk.STATE_NORMAL, self._fg_col)
        #calc direction and gap
        gap = ls_index - self._ls_idx
        #print "ls_index=", ls_index, "  gap=", gap
        if ls_index > self._ls_idx:
            #scrolling down
            if self._hl_on_row + gap < self.num_rows - 1:
                self._hl_on_row += gap
            else:
                self._hl_on_row = self.num_rows - 1
        elif ls_index < self._ls_idx:
            #scrolling up
            if self._hl_on_row + gap > 0:
                self._hl_on_row += gap
            else:
                self._hl_on_row = 0
        self._ls_idx = ls_index
        self._update_display()
        #callback?
        if self.changed_cb is not None:
            self.changed_cb()

    def modify_font(self, pango_font_desc):
        """set list font"""
        for i in range(self.num_rows):
            self.rows[i][1].modify_font(pango_font_desc)

    def modify_highlight_bg(self, gtk_state, hl_bg_col):
        """set list highlighted bar colour"""
        self._hl_bg_col = hl_bg_col
        self.scroll_top.modify_bg(gtk_state, self._hl_bg_col)
        self.scroll_bottom.modify_bg(gtk_state, self._hl_bg_col)
        for i in range(self.num_rows):
            self.rows[i][0].modify_bg(gtk_state, self._hl_bg_col)

    def modify_highlight_fg(self, gtk_state, hl_fg_col):
        """set list highlighted text colour"""
        self._hl_fg_col = hl_fg_col

    def modify_fg(self, gtk_state, fg_col):
        """set list foreground colour"""
        self._fg_col = fg_col
        self.scroll_top.modify_fg(gtk_state, self._fg_col)
        self.scroll_bottom.modify_fg(gtk_state, self._fg_col)
        for i in range(self.num_rows):
            self.rows[i][1].modify_fg(gtk_state, self._fg_col)

    def set_list_size(self, rows):
        """sets number of rows in scrolled list"""
        #remove existing rows
        if self.num_rows > 0:
            for i in range(self.num_rows):
                self.fixd.remove(self.rows[i][0])
        #set number rows
        self.num_rows = rows
        self._row_height = self.height / self.num_rows
        self._hl_on_row = 0
        #create list
        self._create_list_labels()

    def set_property(self, property_name, property_value):
        """set property"""
        pass

    def set_size_request(self, width, height):
        """change size"""
        self.width, self.height = width, height
        #set list "container" size
        self.fixd.set_size_request(width, height)
        self.scroll_top.set_size_request(width, 15)
        self.scroll_bottom.set_size_request(width, 15)
        self.fixd.move(self.scroll_top, 0, 0)
        self.fixd.move(self.scroll_bottom, 0, self.height - 15)
        #set list rows size
        self._row_height = (self.height - 30) / self.num_rows
        for i in range(self.num_rows):
            self.rows[i][0].set_size_request(self.width, self._row_height)
            self.fixd.move(self.rows[i][0], 0, 15 + (i * self._row_height))

    def _create_container(self):
        """create list container"""
        #create fixed layout to put list labels on
        self.fixd = gtk.Fixed()
        self.fixd.set_size_request(self.width, self.height)
        self.fixd.show()
        #create top & bottom scroll arrows
        self.scroll_top = gtk.Arrow(gtk.ARROW_UP, gtk.SHADOW_NONE)
        self.scroll_bottom = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_NONE)
        self.scroll_top.show()
        self.scroll_bottom.show()
        self.scroll_top.set_alignment(0.5, 1.0)
        self.scroll_bottom.set_alignment(0.5, 0.0)
        self.scroll_top.set_property('visible', False)
        self.scroll_bottom.set_property('visible', False)
        self.fixd.add(self.scroll_top)
        self.fixd.add(self.scroll_bottom)
        self.fixd.move(self.scroll_top, 0, 0)
        self.fixd.move(self.scroll_bottom, 0, self.height - 15)
        #add to parent
        if self.parent is not None:
            self.parent.add(self.fixd)

    def _create_list_labels(self):
        """create labels as rows"""
        #create labels
        self.rows = []
        for i in range(self.num_rows):
            eb = gtk.EventBox()
            lbl = gtk.Label()
            lbl.show()
            eb.add(lbl)
            eb.show()
            #eb.set_size_request(self.width, self._row_height)
            eb.set_visible_window(False)
            lbl.set_size_request(self.width, self._row_height)
            self.fixd.add(eb)
            #self.fixd.move(eb, 0, (i * self._row_height))
            self.rows.append((eb, lbl))
        #colours
        self.modify_fg(gtk.STATE_NORMAL, self._fg_col)
        self.modify_highlight_bg(gtk.STATE_NORMAL, self._hl_bg_col)

    def _update_display(self):
        """display the list in the correct position"""
        #calc item that should be at top of list
        top_ls_idx = self._ls_idx - self._hl_on_row
        #shift highlighted rows?
        #print "top_ls_idx=", top_ls_idx, "   _ls_idx=", self._ls_idx, "   _hl_on_row=", self._hl_on_row, "  num_rows=", self.num_rows, "  len=", len(self.ls)
        if self.num_rows > (len(self.ls) - 1):
            #should be able to show whole list at once
            top_ls_idx = 0
            self._hl_on_row = self._ls_idx
        elif (top_ls_idx + self.num_rows) > len(self.ls):
            #shift list down
            self._hl_on_row = ((top_ls_idx + self.num_rows) - (len(self.ls) - 1))
            top_ls_idx = self._ls_idx - self._hl_on_row
        #display scroll limiters?
        if self.display_limiters:
            if top_ls_idx < 1:
                #can't scroll up any more
                self.scroll_top.set_property('visible', False)
            else:
                self.scroll_top.set_property('visible', True)
            if top_ls_idx > (len(self.ls) - self.num_rows -1):
                #can't scroll down any more
                self.scroll_bottom.set_property('visible', False)
            else:
                self.scroll_bottom.set_property('visible', True)
        #set highlighted row
        self.rows[self._hl_on_row][0].set_visible_window(True)
        self.rows[self._hl_on_row][1].modify_fg(gtk.STATE_NORMAL, self._hl_fg_col)
        #populate rows in list
        for i in range(self.num_rows):
            if top_ls_idx + i > len(self.ls) - 1:
                self.rows[i][1].set_text('')
            else:
                self.rows[i][1].set_text(self.ls[top_ls_idx + i])

