#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        wc_common.py
# Description: common code to wahcade windows
# Copyright (c) 2006   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import os
import sys

#gtk
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import pango

#project modules
from constants import *

class WahCade:
    def hide_mouse_cursor(self, win):
        """hide mouse cursor"""
        pix_data = """/* XPM */
            static char * invisible_xpm[] = {
            "1 1 1 1",
            "       c None",
            " "};"""
        color = gtk.gdk.Color()
        pix = gtk.gdk.pixmap_create_from_data(None, pix_data, 1, 1, 1, color, color)
        invisible = gtk.gdk.Cursor(pix, pix, color, color, 0, 0)
        win.window.set_cursor(invisible)

    def get_layout_item_properties(self, lines, offset):
        """get properties for item in layout"""
        d={}
        d['visible'] = (lines[offset].lower() == 'true')
        d['transparent'] = (lines[offset + 1] == '0')
        d['background-col'] = self.get_colour(int(lines[offset + 2]))
        d['text-col'] = self.get_colour(int(lines[offset + 3]))
        d['font'] = lines[offset + 4]
        d['font-bold'] = (lines[offset + 5].lower() == 'true')
        d['font-italic'] = (lines[offset + 6].lower() == 'true')
        d['font-size'] = float(lines[offset + 7])
        d['text-align'] = int(lines[offset + 8])
        d['x'] = int(lines[offset + 9])
        d['y'] = int(lines[offset + 10])
        d['width'] = int(lines[offset + 11])
        d['height'] = int(lines[offset + 12])
        #done
        return d

    def get_colour(self, col):
        """convert decimal colour into format suitable for gtk colour"""
        hex_col = hex(col)[2:].rjust(6, '0').upper()
        #re-arrange
        hex_col = '#%s%s%s' % (hex_col[4:6], hex_col[2:4], hex_col[0:2])
        return hex_col

    def set_list_height(self, scw, tvw, original_size):
        """set height of given scolled window to given list"""
        scw_size = scw.size_request()
        tvw_size = tvw.size_request()
        if tvw_size[1] < scw_size[1]:
            #smaller, so resizing
            scw.set_size_request(original_size[0], tvw_size[1])
        else:
            #make full size
            scw.set_size_request(original_size[0], original_size[1])

    def get_game_image(self, img_path, game_info):
        """check various permutations of file name and return img filename"""
        #image formats
        image_formats = ['png', 'jpg']
        for img_format in image_formats:
            #check upper & lower case rom name & clone_of names, etc
            img_filename = os.path.join(img_path, '%s.%s' % (game_info['rom_name'], img_format))
            if not os.path.exists(img_filename):
                img_filename = os.path.join(img_path, '%s.%s' % (game_info['rom_name'], img_format)).lower()
                if not os.path.exists(img_filename):
                    img_filename = os.path.join(img_path, '%s.%s' % (game_info['clone_of'], img_format))
                    if not os.path.exists(img_filename):
                        img_filename = os.path.join(img_path, '%s.%s' % (game_info['clone_of'], img_format)).lower()
            if os.path.exists(img_filename):
                break
        return img_filename

    def get_scaled_pixbuf(self, img, img_filename, keep_aspect):
        """get a pixbuf with image from filename (scaled to fit)"""
        #get image widget size
        img_width, img_height = img.size_request()
        #load image / empty image
        if os.path.exists(img_filename):
            #load image
            pb = gtk.gdk.pixbuf_new_from_file(img_filename)
        else:
            #load empty image
            pb = gtk.gdk.pixbuf_new_from_file(os.path.join(sys.path[0], 'pixmaps', 'empty.png'))
        #calc image scale
        img_scale_x = float(img_width) / float(pb.get_width())
        img_scale_y = float(img_height) / float(pb.get_height())
        #scale to width or height
        if keep_aspect:
            if (pb.get_height()) * img_scale_x > img_height:
                #scale to height
                new_pb_width = int(float(pb.get_width()) * img_scale_y)
                new_pb_height = int(float(pb.get_height()) * img_scale_y)
            else:
                #scale to width
                new_pb_width = int(float(pb.get_width()) * img_scale_x)
                new_pb_height = int(float(pb.get_height()) * img_scale_x)
        else:
            new_pb_width = img_width
            new_pb_height = img_height
        #scale artwork to display size
        scaled_pb = pb.scale_simple(new_pb_width, new_pb_height, gtk.gdk.INTERP_HYPER)
        #set video width & height
        if img == self.video_artwork_widget:
            self.video_width = new_pb_width
            self.video_height = new_pb_height
        #done
        return scaled_pb

    def display_scaled_image(self, img, img_filename, keep_aspect):
        """load given image widget with filename (scaled to fit)"""
        pb = self.get_scaled_pixbuf(img, img_filename, keep_aspect)
        img.set_from_pixbuf(pb)

    def do_events(self):
        """process any outstanding gtk events"""
        while gtk.events_pending():
            gtk.main_iteration(False)

    def get_path(self, path):
        """return a given path, with user expansion"""
               #is this a directory? expand user name if so
        if path:
            if os.path.exists(os.path.expanduser(path)):
                path = os.path.expanduser(path)
        #done
        return path
