# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        win_history.py
# Description: history viewer file
# Copyright (c) 2006   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import os
import sys

#gtk
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import pango

#project modules
from constants import *
from wc_common import WahCade
from mamewah_ini import MameWahIni
from scrolled_list import ScrollList

class WinHistory(WahCade):
    """History Window"""
    def __init__(self, WinMain):
        #set main window
        self.WinMain = WinMain
        #open history viewer ini
        self.histview_ini = MameWahIni(os.path.join(self.WinMain.config_dir, 'histview.ini'))
        if not os.path.exists(self.histview_ini.get('history_dat_file')):
            print "Error: history file: [%s] does not exist" % (
                self.histview_ini.get('history_dat_file'))
            self.winHistory = None
            return
        #build the window
        self.winHistory = gtk.Window()
        self.fixd = gtk.Fixed()
        self.imgBackground = gtk.Image()
        self.lblHeading = gtk.Label()
        self.sclHistory = ScrollList()
        self.fixd.add(self.imgBackground)
        self.fixd.add(self.lblHeading)
        self.fixd.add(self.sclHistory.fixd)
        self.winHistory.add(self.fixd)
        self.imgBackground.show()
        self.lblHeading.show()
        self.fixd.show()
        self.winHistory.set_decorated(False)
        self.winHistory.set_transient_for(WinMain.winMain)
        self.winHistory.set_modal(True)
        self.winHistory.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.winHistory.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
        #build list
        self.lsHistory = []
        #get keyboard & mouse events
        self.winHistory.add_events(gtk.gdk.POINTER_MOTION_MASK | gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK)
        self.winHistory.connect("key_press_event", self.WinMain.on_winMain_key_press)
        self.winHistory.connect("key_release_event", self.WinMain.on_winMain_key_press)
        self.winHistory.connect("motion_notify_event", self.WinMain.on_winMain_key_press)
        #
        self.current_line = 0
        self.old_line = -1
        #hide mouse
        self.winHistory.show()
        if not self.WinMain.wahcade_ini.getint('show_cursor'):
            self.hide_mouse_cursor(self.winHistory)
        #get history
        self.history = self.read_history(self.histview_ini.get('history_dat_file'))
        #get fonts
        heading_fd = pango.FontDescription(self.histview_ini.get('title_font'))
        history_fd = pango.FontDescription(self.histview_ini.get('history_font'))
        history_font_size = pango.PIXELS(history_fd.get_size())
        #setup window
        main_width, main_height = self.WinMain.winMain.size_request()
        hvwr_width = int(main_width * 0.66)
        hvwr_height = int(main_height * 0.66)
        self.winHistory.set_size_request(hvwr_width, hvwr_height)
        self.winHistory.set_default_size(hvwr_width, hvwr_height)
        self.winHistory.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('black'))
        self.fixd.move(self.imgBackground, 0, 0)
        self.imgBackground.set_size_request(hvwr_width, hvwr_height)
        back_img = os.path.expanduser(self.histview_ini.get('background_image'))
        if os.path.exists(back_img):
            self.imgBackground.set_from_file(back_img)
        self.fixd.move(self.lblHeading, 25, 5)
        self.lblHeading.set_size_request(hvwr_width - 50, 20)
        self.lblHeading.modify_font(heading_fd)
        self.lblHeading.set_property('visible', True)
        self.lblHeading.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('yellow'))
        self.fixd.move(self.sclHistory.fixd, 25, 35)
        #setup history list
        self.sclHistory.set_list_size(int((hvwr_height - 75) / (history_font_size * 2)))
        self.sclHistory.set_size_request(hvwr_width - 50, hvwr_height - 75)
        self.sclHistory.modify_font(history_fd)
        self.sclHistory.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('yellow'))
        self.sclHistory.modify_highlight_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('blue'))
        self.sclHistory.modify_highlight_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('yellow'))
        #setup font info for history list
        context = self.sclHistory.rows[0][1].get_pango_context()
        metrics = context.get_metrics(history_fd)
        char_width = pango.PIXELS(metrics.get_approximate_char_width())
        self.line_length = ((hvwr_width - 50) / (char_width + 2))

    def set_history(self, rom_name):
        """display history for rom_name"""
        if not self.history.has_key(rom_name):
            return
        #display
        self.lsHistory = []
        for line in self.history[rom_name]:
            while len(line) > self.line_length:
                ll = self.line_length
                while line[ll] != ' ' and ll > 20:
                    ll -= 1
                self.lsHistory.append(line[:ll])
                line = line[ll + 1:]
            self.lsHistory.append(line)
        self.sclHistory.ls = self.lsHistory
        self.lblHeading.set_text(rom_name)
        self.current_line = 0
        self.sclHistory.set_selected(self.current_line)
        self.winHistory.show()

    def read_history(self, dat_filename):
        """read history into dictionary"""
        f = open(dat_filename, 'r')
        d = {}
        while True:
            try:
                line = f.next().strip()
            except StopIteration:
                break
            #start of a game history
            if line[:5] == '$info':
                #history can be for more than one rom
                rom_names = line[6:-1].split(',')
                hist_txt = []
                #read file until end of current game history
                while line != '$end':
                    try:
                        line = f.next().strip()
                    except StopIteration:
                        line = '$end'
                    #add blank lines
                    if line == '':
                        line = ' '
                    if line[0] != '$':
                        hist_txt.append(line)
                if hist_txt != []:
                    for rom_name in rom_names:
                        d[rom_name.upper()] = hist_txt
        #done
        return d

