# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        wahcade.py
# Description: Main Window
# Copyright (c) 2006   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import os
import sys
import shutil
import glob
import random
import time
from operator import itemgetter
from subprocess import Popen

#gtk
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import pango

#project modules
from constants import *
from scrolled_list import ScrollList
from key_consts import mamewah_keys
from wc_common import WahCade
from win_options import WinOptions
from win_message import WinMessage
from win_scrsaver import WinScrSaver
from win_history import WinHistory
from win_cpviewer import get_viewer_window
import filters
from mamewah_ini import MameWahIni
if sys.platform != 'win32':
    try:
        from mplayer import Mplayer
    except:
        pass

class WinMain(WahCade):
    """Wah!Cade Main Window"""
    def __init__(self, config_opts):
        #command-line options
        self.config_opts = config_opts
        if self.config_opts.use_app_config or sys.platform == 'win32':
            #application config
            self.config_dir = os.path.join(sys.path[0], 'config')
        else:
            #default, user config
            if not os.path.exists(os.path.expanduser('~/.wahcade')):
                #copy config over
                shutil.copytree(
                    os.path.join(sys.path[0], 'config.dist'),
                    os.path.expanduser('~/.wahcade'))
                #now we've copied stuff, quit and tell the user
                print "Wah!Cade config files created in: [%s]" % (os.path.expanduser('~/.wahcade'))
                print "Restart after you've configuring (see the README file)."
                sys.exit(0)
            else:
                #got with the default config location
                self.config_dir = os.path.expanduser('~/.wahcade')
        #build the window
        self.winMain = gtk.Window()
        self.fixd = gtk.Fixed()
        self.imgBackground = gtk.Image()
        self.imgMainLogo = gtk.Image()
        self.lblGameListIndicator = gtk.Label()
        self.lblEmulatorName = gtk.Label()
        self.lblGameSelected = gtk.Label()
        self.drwVideo = gtk.DrawingArea()
        self.imgArtwork1 = gtk.Image()
        self.imgArtwork2 = gtk.Image()
        self.imgArtwork3 = gtk.Image()
        self.imgArtwork4 = gtk.Image()
        self.imgArtwork5 = gtk.Image()
        self.imgArtwork6 = gtk.Image()
        self.imgArtwork7 = gtk.Image()
        self.imgArtwork8 = gtk.Image()
        self.imgArtwork9 = gtk.Image()
        self.imgArtwork10 = gtk.Image()
        self.lblGameDescription = gtk.Label()
        self.lblRomName = gtk.Label()
        self.lblYearManufacturer = gtk.Label()
        self.lblScreenType = gtk.Label()
        self.lblControllerType = gtk.Label()
        self.lblDriverStatus = gtk.Label()
        self.lblCatVer = gtk.Label()
        #create scroll list widget
        self.sclGames = ScrollList()
        #image & label lists
        self._layout_items = [
            (8, self.imgMainLogo),
            (21, self.lblGameListIndicator),
            (34, self.lblEmulatorName),
            (47, self.sclGames),
            (60, self.lblGameSelected),
            (73, self.imgArtwork1),
            (86, self.imgArtwork2),
            (99, self.imgArtwork3),
            (112, self.imgArtwork4),
            (125, self.imgArtwork5),
            (138, self.imgArtwork6),
            (151, self.imgArtwork7),
            (164, self.imgArtwork8),
            (177, self.imgArtwork9),
            (190, self.imgArtwork10),
            (203, self.lblGameDescription),
            (216, self.lblRomName),
            (229, self.lblYearManufacturer),
            (242, self.lblScreenType),
            (255, self.lblControllerType),
            (268, self.lblDriverStatus),
            (281, self.lblCatVer)]
        self._main_images = [
            self.imgArtwork1,
            self.imgArtwork2,
            self.imgArtwork3,
            self.imgArtwork4,
            self.imgArtwork5,
            self.imgArtwork6,
            self.imgArtwork7,
            self.imgArtwork8,
            self.imgArtwork9,
            self.imgArtwork10]
        #add widgets to main window
        self.fixd.add(self.imgBackground)
        self.imgBackground.show()
        self.fixd.show()
        self.winMain.add(self.fixd)
        for line, widget in self._layout_items:
            if widget != self.sclGames:
                self.fixd.add(widget)
                widget.show()
            else:
                self.fixd.add(widget.fixd)
        #video widget
        self.video_playing = False
        self.video_enabled = False
        self.video_timer = None
        self.drwVideo.connect_after("realize", self.on_drwVideo_realize)
        self.drwVideo.show()
        self.fixd.add(self.drwVideo)
        #set window properties
        self.winMain.set_position(gtk.WIN_POS_NONE)
        self.winMain.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_NORMAL)
        self.winMain.set_title('Wah!Cade')
        #init random engine
        random.seed()
        #build list
        self.lsGames = []
        #timers
        self.scrsave_time = time.time()
        self.start_timer('scrsave')
        #get keyboard & mouse events
        self.winMain.connect("destroy", self.on_winMain_destroy)
        self.winMain.add_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK) #gtk.gdk.POINTER_MOTION_MASK |
        self.winMain.connect("key_press_event", self.on_winMain_key_press)
        self.winMain.connect("key_release_event", self.on_winMain_key_press)
        self.winMain.connect("motion_notify_event", self.on_winMain_key_press)
        #get ini files
        self.wahcade_ini = MameWahIni(os.path.join(self.config_dir, 'wahcade.ini'))
        self.current_emu = self.wahcade_ini.get('current_emulator')
        self.ctrlr_ini = MameWahIni(os.path.join(self.config_dir, 'ctrlr', 'default.ini'), 'ctrlr')
        self.keep_aspect = self.wahcade_ini.getint('keep_image_aspect')
        #create options window
        self.options = WinOptions(self)
        self.options.winOptions.hide()
        #create message window
        self.message = WinMessage(self)
        self.message.winMessage.hide()
        #create screen saver window
        self.scrsaver = WinScrSaver(self)
        self.scrsaver.winScrSaver.hide()
        self.scrsave_timer = None
        #add options, message & screen saver widgets to _layout_items
        self._layout_items += [
            (301, self.options.lblHeading),
            (314, self.options.sclOptions),
            (327, self.options.lblSettingHeading),
            (340, self.options.lblSettingValue),
            (357, self.message.lblHeading),
            (370, self.message.lblMessage),
            (383, self.message.lblPrompt),
            (396, self.scrsaver.imgArtwork1),
            (409, self.scrsaver.imgArtwork2),
            (422, self.scrsaver.imgArtwork3),
            (435, self.scrsaver.imgArtwork4),
            (448, self.scrsaver.imgArtwork5),
            (461, self.scrsaver.imgArtwork6),
            (474, self.scrsaver.imgArtwork7),
            (487, self.scrsaver.imgArtwork8),
            (500, self.scrsaver.imgArtwork9),
            (513, self.scrsaver.imgArtwork10),
            (526, self.scrsaver.lblGameDescription),
            (539, self.scrsaver.lblMP3Name)]
        #build list of emulators
        self.emu_lists = []
        emu_ini_files = glob.glob(os.path.join(self.config_dir, 'ini', '*[!0-9].ini'))
        for emu_ini in emu_ini_files:
            ini = MameWahIni(emu_ini)
            self.emu_lists.append(
                [ini.get('emulator_title'),
                os.path.splitext(os.path.basename(emu_ini))[0]])
        self.emu_lists.sort()
        #load list
        self.current_list_ini = None
        self.emu_ini = None
        self.load_emulator()
        #fullscreen? and hide cursor & video widget
        self.winMain.show()
        self.drwVideo.set_property('visible', False)
        if not self.wahcade_ini.getint('show_cursor'):
            self.hide_mouse_cursor(self.winMain)
        if self.config_opts.fullscreen:
            self.winMain.fullscreen()

    def on_drwVideo_realize(self, drawingarea):
        """video widget initialise"""
        drawingarea.modify_bg(gtk.STATE_NORMAL, drawingarea.style.black)
        #dispay vid
        self.video_xid = drawingarea.window.xid
        #print "self.video_xid =", self.video_xid
        stylee = drawingarea.get_style()
        self.gc = stylee.fg_gc[gtk.STATE_NORMAL]
        self.drwVideo.set_property('visible', True)

    def on_winMain_destroy(self, *args):
        """done, quit the application"""
        #save ini files
        self.wahcade_ini.write()
        self.emu_ini.write()
        self.current_list_ini.write()
        #write fav list
        filters.write_fav_list(
            os.path.join(self.config_dir, 'files', '%s.fav' % (self.current_emu)),
            self.emu_favs_list)
        #exit gtk loop
        gtk.main_quit()

    def on_winMain_key_press(self, widget, event, *args):
        """key pressed - translate to mamewah setup"""
        mw_keys = []
        mw_key = ''
        mw_func = ''
        #reset screen saver time (and turn off if necessary)
        self.scrsave_time = time.time()
        if self.scrsaver.running:
            self.scrsaver.running = False
            self.scrsaver.winScrSaver.hide()
            self.start_timer('scrsave')
            self.start_timer('video')
        #if event.type == gtk.gdk.MOTION_NOTIFY:
            #mouse moved
        if event.type == gtk.gdk.KEY_PRESS:
            #keyboard pressed, get gtk keyname
            keyname = gtk.gdk.keyval_name(event.keyval).lower()
            if not mamewah_keys.has_key(keyname):
                return
            #get mamewah keyname
            mw_keys = mamewah_keys[keyname]
        elif event.type == gtk.gdk.KEY_RELEASE:
            #keyboard released, update labels, images, etc
            if self.old_game_idx != self.current_game_idx:
                self.on_sclGames_changed()
            if self.options.old_option_idx != self.options.current_option_idx:
                self.options.on_sclOptions_changed()
            #remember current game
            self.old_game_idx = self.current_game_idx
            self.options.old_option_idx = self.options.current_option_idx
            if self.histview:
                self.histview.old_line = self.histview.current_line
        #got something?
        if mw_keys == []:
            return
        #get mamewah function from key
        for mw_key in mw_keys:
            mw_functions = self.ctrlr_ini.reverse_get(mw_key)
            if mw_functions:
                break
        #print "mw_key=", mw_key, "   mw_functions=", mw_functions
        for mw_func in mw_functions:
            #which function?
            if widget == self.winMain:
                #main form
                if mw_func == 'UP_1_GAME':
                    if self.current_game_idx > 0:
                        self.current_game_idx -= 1
                elif mw_func == 'DOWN_1_GAME':
                    if self.current_game_idx < len(self.lsGames) - 1:
                        self.current_game_idx += 1
                elif mw_func == 'UP_1_PAGE':
                    if self.current_game_idx > self.sclGames.num_rows:
                        self.current_game_idx -= self.sclGames.num_rows
                    else:
                        self.current_game_idx = 0
                elif mw_func == 'DOWN_1_PAGE':
                    if self.current_game_idx < len(self.lsGames) - self.sclGames.num_rows:
                        self.current_game_idx += self.sclGames.num_rows
                    else:
                        self.current_game_idx = len(self.lsGames) - 1
                elif mw_func == 'UP_1_LETTER':
                    cl = self.lsGames[self.current_game_idx][0][0]
                    games = [r[0] for r in self.lsGames]
                    games = games[:self.current_game_idx]
                    games.reverse()
                    i = 0
                    for row in games:
                        i += 1
                        if row[0] != cl:
                            self.current_game_idx -= i
                            break
                elif mw_func == 'DOWN_1_LETTER':
                    cl = self.lsGames[self.current_game_idx][0][0]
                    games = [r[0] for r in self.lsGames]
                    games = games[self.current_game_idx:]
                    i = -1
                    for row in games:
                        i += 1
                        if row[0] != cl:
                            self.current_game_idx += i
                            break
                elif mw_func == 'RANDOM_GAME':
                    self.current_game_idx = self.select_random_game()
                elif mw_func == 'LAUNCH_GAME':
                    self.launch_game()
                elif mw_func == 'MENU_SHOW':
                    self.options.set_menu('main')
                    self.options.winOptions.show()
                elif mw_func == 'NEXT_EMULATOR':
                    emu_list = [e[1] for e in self.emu_lists]
                    emu_idx = emu_list.index(self.current_emu)
                    if emu_idx < len(emu_list) - 1:
                        emu_idx += 1
                    else:
                        emu_idx = 0
                    self.load_emulator(emu_list[emu_idx])
                elif mw_func == 'PREVIOUS_EMULATOR':
                    emu_list = [e[1] for e in self.emu_lists]
                    emu_idx = emu_list.index(self.current_emu)
                    if emu_idx > 0:
                        emu_idx -= 1
                    else:
                        emu_idx = len(emu_list) - 1
                    self.load_emulator(emu_list[emu_idx])
                elif mw_func == 'NEXT_GAMELIST':
                    self.old_game_idx = 0
                    if self.current_list_idx < len(self.game_lists) - 1:
                        self.current_list_idx += 1
                    else:
                        self.current_list_idx = 0
                    self.load_list()
                elif mw_func == 'PREVIOUS_GAMELIST':
                    self.old_game_idx = 0
                    if self.current_list_idx > 0:
                        self.current_list_idx -= 1
                    else:
                        self.current_list_idx = len(self.game_lists) - 1
                    self.load_list()
                elif mw_func == 'LAUNCH_APP_1':
                    self.launch_external_application(1)
                elif mw_func == 'LAUNCH_APP_2':
                    self.launch_external_application(2)
                elif mw_func == 'LAUNCH_APP_3':
                    self.launch_external_application(3)
                elif mw_func == 'EXIT_TO_WINDOWS':
                    self.on_winMain_destroy()
            if widget == self.options.winOptions:
                #options form
                if mw_func == 'OP_UP_1_OPTION':
                    if self.options.current_option_idx> 0:
                        self.options.current_option_idx-= 1
                elif mw_func == 'OP_DOWN_1_OPTION':
                    if self.options.current_option_idx< len(self.options.lsOptions) - 1:
                        self.options.current_option_idx+= 1
                elif mw_func == 'OP_UP_1_OPTION_PAGE':
                    if self.options.current_option_idx> self.options.sclOptions.num_rows:
                        self.options.current_option_idx-= self.options.sclOptions.num_rows
                    else:
                        self.options.current_option_idx= 0
                elif mw_func == 'OP_DOWN_1_OPTION_PAGE':
                    if self.options.current_option_idx< len(self.options.lsOptions) - self.options.sclOptions.num_rows:
                        self.options.current_option_idx+= self.options.sclOptions.num_rows
                    else:
                        self.options.current_option_idx= len(self.options.lsOptions) - 1
                elif mw_func == 'OP_MENU_SELECT':
                    self.options.menu_selected()
                elif mw_func == 'OP_MENU_BACK':
                    if self.options.current_menu == 'main':
                        self.options.winOptions.hide()
                    elif self.options.current_menu in ['emu_list', 'game_list', 'list_options']:
                        self.options.set_menu('main')
                    elif self.options.current_menu == 'add_to_list':
                        self.options.set_menu('list_options')
                    elif self.options.current_menu == 'generate_ftr':
                        if self.current_filter_changed:
                            #generate new filter
                            self.options.set_menu('generate_new_list')
                        else:
                            self.options.set_menu('list_options')
                    elif self.options.current_menu.startswith('ftr:'):
                        self.options.set_menu('generate_ftr')
                    elif self.options.current_menu == 'find':
                        self.options.find_game('back')
            #which function?
            if self.histview:
                if widget == self.histview.winHistory:
                    #history viewer
                    if mw_func in ['UP_1_GAME', 'OP_UP_1_OPTION']:
                        if self.histview.current_line > 0:
                            self.histview.current_line -= 1
                    elif mw_func in ['DOWN_1_GAME', 'OP_DOWN_1_OPTION']:
                        if self.histview.current_line < len(self.histview.lsHistory) - 1:
                            self.histview.current_line += 1
                    elif mw_func in ['UP_1_PAGE', 'UP_1_LETTER', 'OP_UP_1_OPTION_PAGE']:
                        if self.histview.current_line > self.histview.sclHistory.num_rows:
                            self.histview.current_line -= self.histview.sclHistory.num_rows
                        else:
                            self.histview.current_line = 0
                    elif mw_func in ['DOWN_1_PAGE', 'DOWN_1_LETTER', 'OP_DOWN_1_OPTION_PAGE']:
                        if self.histview.current_line < len(self.histview.lsHistory) - self.histview.sclHistory.num_rows:
                            self.histview.current_line += self.histview.sclHistory.num_rows
                        else:
                            self.histview.current_line = len(self.histview.lsHistory) - 1
                    elif mw_func in ['EXIT_TO_WINDOWS', 'LAUNCH_APP_1']:
                        self.histview.winHistory.hide()
            #which function?
            if self.cpviewer:
                if widget == self.cpviewer.winCPViewer:
                    if mw_func in ['EXIT_TO_WINDOWS', 'LAUNCH_APP_2']:
                        self.cpviewer.winCPViewer.hide()
        #game /list changed?
        if self.current_game_idx != self.old_game_idx:
            self.sclGames.set_selected(self.current_game_idx)
            self.current_list_ini.set('current_game', self.current_game_idx)
        if self.options.current_option_idx!= self.options.old_option_idx:
            self.options.sclOptions.set_selected(self.options.current_option_idx)
        if self.histview:
            if self.histview.current_line!= self.histview.old_line:
                self.histview.sclHistory.set_selected(self.histview.current_line)

    def on_sclGames_changed(self, *args):
        """game selected"""
        #print "self.current_game_idx=", self.current_game_idx, "   len=", len(self.lsGames)
        if self.current_game_idx > len(self.lsGames) - 1:
            #**FIXME** should blank labels & images here...
            return
        #get info
        game_info = filters.get_game_dict(self.lsGames[self.current_game_idx])
        #set layout text items
        self.lblGameDescription.set_text(game_info['game_name'])
        self.lblGameSelected.set_text('Game %s of %s selected' % (
            self.current_game_idx + 1,
            len(self.lsGames)))
        if game_info['clone_of'] != '':
            rom_name_desc = '%s (Clone of %s)' % (game_info['rom_name'], game_info['clone_of'])
        else:
            rom_name_desc = game_info['rom_name']
        self.lblRomName.set_text(rom_name_desc)
        self.lblYearManufacturer.set_text('%s %s' % (game_info['year'], game_info['manufacturer']))
        self.lblScreenType.set_text('%s %s' % (game_info['screen_type'], game_info['display_type']))
        self.lblControllerType.set_text(game_info['controller_type'])
        self.lblDriverStatus.set_text('%s, %s, %s' % (
            game_info['driver_status'],
            game_info['colour_status'],
            game_info['sound_status']))
        self.lblCatVer.set_text(game_info['category'])
        #start video timer
        self.start_timer('video')
        #set layout images
        for i, img in enumerate(self._main_images):
            img_path = self.emu_ini.get('artwork_%s_image_path' % (i + 1))
            if img_path and img.get_property('visible'):
                img_filename = self.get_game_image(img_path, game_info)
                self.display_scaled_image(img, img_filename, self.keep_aspect)

    def on_scrsave_timer(self):
        """timer event - check to see if we need to start video or screen saver"""
        #need to start screen saver?
        if int(time.time() - self.scrsave_time) >= self.wahcade_ini.getint('delay'):
            #yes
            if self.emu_ini.get('saver_type') in ['blank_screen', 'slideshow']:
                #stop any vids playing
                if self.video_playing:
                    self.video_playing = False
                    self.mplayer.close()
                    self.drwVideo.set_property('visible', False)
                #start screen saver
                self.scrsaver.start_scrsaver(self.emu_ini.get('saver_type'))
            else:
                print "Error: wrong screen saver type: [%s]" % (self.emu_ini.get('saver_type'))
            return False
        #done
        return True

    def on_video_timer(self):
        """timer event - start video playing"""
        #start video playing?
        if self.video_enabled:
            if not self.video_playing:
                #get info
                rom_name = self.lsGames[self.current_game_idx][GL_ROM_NAME]
                vid_filename = os.path.join(
                    self.emu_ini.get('movie_preview_path'),
                    '%s.avi' % (rom_name.lower()))
                if os.path.exists(vid_filename):
                    #resize video vidget
                    self.video_playing = True
                    img_w, img_h = self.video_artwork_widget.get_size_request()
                    xpos = (img_w - self.video_width) / 2
                    self.fixd.move(
                        self.drwVideo,
                        self.fixd.child_get_property(self.video_artwork_widget, 'x') + xpos,
                        self.fixd.child_get_property(self.drwVideo, 'y'))
                    self.drwVideo.set_size_request(self.video_width, self.video_height)
                    self.drwVideo.set_property('visible', True)
                    #start mplayer
                    self.mplayer = Mplayer(
                        self.video_xid,
                        (self.wahcade_ini.getint('movie_volume') == 0))
                    self.mplayer.play(vid_filename)
        #done
        return False

    def launch_game(self):
        """run currently selected game"""
        #stop any vids playing
        if self.video_playing:
            self.video_playing = False
            self.mplayer.close()
            self.drwVideo.set_property('visible', False)
        #launch external apps
        #external_apps = self.emu_ini.get('auto_launch_apps').split(',')
        #[self.launch_external_application(app_num) for app_num in external_apps]
        #get rom name
        rom = self.lsGames[self.current_game_idx][GL_ROM_NAME]
        #show launch message
        self.message.display_message(
                'Starting...',
                '%s: %s' % (rom, self.lsGames[self.current_game_idx][GL_GAME_NAME]))
        #start timing
        time_game_start = time.time()
        #wait a bit - to let message window display
        #print "time_game_start=", time_game_start, time.time()
        while True:
            self.do_events()
            if (time_game_start + 0.1) < time.time():
                break
        #get command line options
        opts = self.emu_ini.get('commandline_format')
        opts = opts.replace('[name]', rom)
        opts = opts.replace('[rompath]', self.emu_ini.get('rom_path'))
        opts = opts.replace('[romext]', self.emu_ini.get('rom_extension'))
        #print "opts=", opts
        cmd = "%s %s" % (self.emu_ini.get('emulator_executable'), opts)
        #run emulator & wait for it to finish
        p = Popen(cmd, shell=True)
        sts = os.waitpid(p.pid, 0)
        #hide message window
        self.message.hide()
        self.scrsave_time = time.time()
        #start video timer
        self.start_timer('video')
        #stop timing
        time_game_stop = time.time()
        #add to / update favs list
        if not self.emu_favs_list.has_key(rom):
            self.emu_favs_list[rom] = [
                rom,
                self.lsGames[self.current_game_idx][GL_GAME_NAME],
                0,
                0]
        self.emu_favs_list[rom][FAV_TIMES_PLAYED] += 1
        self.emu_favs_list[rom][FAV_MINS_PLAYED] += int((time_game_stop - time_game_start) / 60)
        #print "time played=", (time_game_stop - time_game_start)

    def launch_external_application(self, app_number):
        """launch app specified in emu.ini"""
        #get app name
        app_name = self.emu_ini.get('app_%s_executable' % (app_number))
        app_params = self.emu_ini.get('app_%s_commandline_format' % (app_number))
        #pre-defined?
        if app_name == 'wahcade-history-viewer' and self.histview:
            #display game history
            self.histview.set_history(self.lsGames[self.current_game_idx][GL_ROM_NAME])
        elif app_name == 'wahcade-cp-viewer' and self.cpviewer:
            #display control panel info
            cpvw_rom = self.lsGames[self.current_game_idx][GL_ROM_NAME]
            #use clone name if necessary
            if self.lsGames[self.current_game_idx][GL_CLONE_OF] != '':
                cpvw_rom = self.lsGames[self.current_game_idx][GL_CLONE_OF]
            self.cpviewer.display_game_details(cpvw_rom)
        else:
            #launch the app
            if os.path.isfile(app_name):
                cmd = '%s %s' % (app_name, app_params)
                print "starting external app: %s" % (cmd)
                os.system(app_name)

    def load_emulator(self, emulator_name=None):
        """load emulator"""
        #load emulator ini file
        if emulator_name:
            self.current_emu = emulator_name
            self.wahcade_ini.set('current_emulator', emulator_name)
        #save current emulator list
        if self.emu_ini:
            self.emu_ini.set('current_list', self.current_list_idx)
            self.emu_ini.write()
        #load new emulator
        self.emu_ini = MameWahIni(os.path.join(self.config_dir, 'ini', '%s.ini' % (self.current_emu)))
        self.lblEmulatorName.set_text(self.emu_ini.get('emulator_title'))
        self.game_lists = []
        self.game_lists_normal = []
        #load layout
        self.layout_file = os.path.join(self.config_dir, 'layouts', self.wahcade_ini.get('layout'), '%s.lay' % (self.current_emu))
        if not os.path.exists(self.layout_file):
            self.layout_file = os.path.join(self.config_dir, 'layouts', self.wahcade_ini.get('layout'), 'layout.lay')
        self.load_layout_file(self.layout_file)
        #set xml file
        filters._mameinfo_file = os.path.join(self.emu_ini.get('dat_file'))
        filters._catver_ini = os.path.join(self.emu_ini.get('catver_ini_file'))
        #calc number of lists
        for i in range(100):
            ini_file = os.path.join(self.config_dir, 'ini', '%s-%s.ini' % (self.current_emu, i))
            if os.path.exists(ini_file):
                ini = MameWahIni(ini_file)
                self.game_lists.append([ini.get('list_title'), i])
                if i != 0 and ini.get('list_type') == 'normal':
                    self.game_lists_normal.append([ini.get('list_title'), i])
            else:
                break
        #load fav list
        fav_name = os.path.join(self.config_dir, 'files', '%s.fav' % (self.current_emu))
        if not os.path.isfile(fav_name):
            #create fav list if it doesn't exist
            f = open(fav_name, 'w')
            f.close()
        self.emu_favs_list = filters.read_fav_list(fav_name)
        #play videos?
        self.check_video_settings()
        #load list
        self.current_list_idx = self.emu_ini.getint('current_list')
        self.load_list()
        #history viewer?
        self.histview = None
        if self.current_emu == 'mame':
            if self.emu_ini.get('app_1_executable') == 'wahcade-history-viewer':
                self.histview = WinHistory(self)
                if self.histview.winHistory:
                    self.histview.winHistory.hide()
                else:
                    self.histview = None
        #cp viewer?
        self.cpviewer = None
        if self.current_emu == 'mame':
            if self.emu_ini.get('app_2_executable') == 'wahcade-cp-viewer':
                self.cpviewer = get_viewer_window(self)
                if self.cpviewer:
                    self.cpviewer.winCPViewer.hide()

    def load_list(self):
        """load current list"""
        #save last list (if defined)
        if self.current_list_ini:
            self.current_list_ini.write()
        #load new list
        self.current_list_ini = MameWahIni(os.path.join(self.config_dir, 'ini', '%s-%s.ini' % (self.current_emu, self.current_list_idx)))
        self.emu_ini.set('current_list', self.current_list_idx)
        #load list & set current game
        self.lblGameListIndicator.set_text(self.current_list_ini.get('list_title'))
        self.current_game_idx = self.current_list_ini.getint('current_game')
        self.old_game_idx = -1
        #has initial lst file been created?
        game_list_file = os.path.join(
            self.config_dir,
            'files',
            '%s-0.lst' % (self.current_emu))
        if not os.path.isfile(game_list_file):
            print "Please Wait. Creating initial filter..."
            self.message.display_message('Please Wait', 'Creating initial filter...')
            self.current_list_idx = 0
            self.current_game_idx = 0
            filters.create_initial_filter(
                filters._mameinfo_file,
                os.path.join(
                    self.config_dir,
                    'files',
                    '%s-0.ftr' % (self.current_emu)),
                game_list_file,
                self.emu_ini)
            self.load_list()
            #hide message
            self.message.hide()
        #load the list of games
        self.pop_games_list()
        #load the list filter
        self.current_filter_changed = False
        if self.current_emu == 'mame':
            filter_file = os.path.join(
                self.config_dir,
                'files',
                '%s-%s.ftr' % (self.current_emu, self.current_list_idx))
            if not os.path.isfile(filter_file):
                #filter doesn't exist, so try and use use filter for list 0
                filter_file = os.path.join(
                    self.config_dir,
                    'files',
                    '%s-0.ftr' % (self.current_emu))
            self.current_filter = filters.read_filter(filter_file)
        else:
            self.current_filter = None

    def load_layout_file(self, layout_filename):
        """load layout file"""
        #read file & strip any crap
        lines = open(layout_filename, 'r').readlines()
        lines = [s.strip() for s in lines]
        lines.insert(0, '.')
        #window sizes
        main_width, main_height = int(lines[1].split(';')[0]), int(lines[2])
        opt_width, opt_height = int(lines[294].split(';')[0]), int(lines[295])
        msg_width, msg_height = int(lines[353].split(';')[0]), int(lines[354])
        #main window
        self.winMain.set_size_request(main_width, main_height)
        self.winMain.set_default_size(main_width, main_height)
        bg_col = gtk.gdk.color_parse(self.get_colour(int(lines[3])))
        self.winMain.modify_bg(gtk.STATE_NORMAL, bg_col)
        self.fixd.move(self.imgBackground, 0, 0)
        self.imgBackground.set_size_request(main_width, main_height)
        if os.path.exists(self.get_path(lines[4])):
            self.imgBackground.set_from_file(self.get_path(lines[4]))
        else:
            self.imgBackground.set_property('visible', False)
        #set options window
        self.options.winOptions.set_size_request(opt_width, opt_height)
        self.options.winOptions.set_default_size(opt_width, opt_height)
        bg_col = gtk.gdk.color_parse(self.get_colour(int(lines[296])))
        self.options.winOptions.modify_bg(gtk.STATE_NORMAL, bg_col)
        self.options.fixd.move(self.options.imgBackground, 0, 0)
        self.options.imgBackground.set_size_request(opt_width, opt_height)
        if os.path.exists(self.get_path(lines[297])):
            self.options.imgBackground.set_from_file(self.get_path(lines[297]))
        else:
            self.options.imgBackground.set_property('visible', False)
        #games list highlight colours
        hl_bg_col = gtk.gdk.color_parse(self.get_colour(int(lines[6])))
        hl_fg_col = gtk.gdk.color_parse(self.get_colour(int(lines[7])))
        self.sclGames.modify_highlight_bg(gtk.STATE_NORMAL, hl_bg_col)
        self.sclGames.modify_highlight_fg(gtk.STATE_NORMAL, hl_fg_col)
        #options list highlight colours
        hl_bg_col = gtk.gdk.color_parse(self.get_colour(int(lines[299])))
        hl_fg_col = gtk.gdk.color_parse(self.get_colour(int(lines[300])))
        self.options.sclOptions.modify_highlight_bg(gtk.STATE_NORMAL, hl_bg_col)
        self.options.sclOptions.modify_highlight_fg(gtk.STATE_NORMAL, hl_fg_col)
        #set message window
        self.message.winMessage.set_size_request(msg_width, msg_height)
        self.message.winMessage.set_default_size(msg_width, msg_height)
        bg_col = gtk.gdk.color_parse(self.get_colour(int(lines[355])))
        self.message.winMessage.modify_bg(gtk.STATE_NORMAL, bg_col)
        self.message.fixd.move(self.message.imgBackground, 0, 0)
        self.message.imgBackground.set_size_request(msg_width, msg_height)
        if os.path.exists(self.get_path(lines[356])):
            self.message.imgBackground.set_from_file(self.get_path(lines[356]))
        else:
            self.message.imgBackground.set_property('visible', False)
        #screen saver window
        self.scrsaver.winScrSaver.set_size_request(main_width, main_height)
        self.scrsaver.winScrSaver.set_default_size(main_width, main_height)
        self.scrsaver.winScrSaver.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('black'))
        #set all window items
        for offset, widget in self._layout_items:
            #get properties
            d = self.get_layout_item_properties(lines, offset)
            #font
            fd = d['font']
            if d['font-bold']:
                fd += ' Bold'
            fd += ' %s' % (d['font-size'])
            font_desc = pango.FontDescription(fd)
            #calc num rows in lists
            if widget in (self.sclGames, self.options.sclOptions):
                widget.set_list_size(int(round(d['height'] / (d['font-size'] * 2)))+1)
            #colours
            fg_col = gtk.gdk.color_parse(d['text-col'])
            bg_col = gtk.gdk.color_parse(d['background-col'])
            widget.modify_font(font_desc)
            widget.modify_fg(gtk.STATE_NORMAL, fg_col)
            #visible?
            widget.set_property('visible', d['visible'])
            #move & size
            widget.set_size_request(d['width'], d['height'])
            if widget == self.sclGames:
                widget = self.sclGames.fixd
            elif widget == self.options.sclOptions:
                widget = self.options.sclOptions.fixd
            #position video widget
            if self.emu_ini.getint('movie_artwork_no') > 0:
                self.video_artwork_widget =  self._main_images[(self.emu_ini.getint('movie_artwork_no') - 1)]
                if widget == self.video_artwork_widget:
                    self.fixd.move(self.drwVideo, d['x'], d['y'])
                    self.drwVideo.set_size_request(d['width'], d['height'])
            #add to fixed layout on correct window
            if offset < 293:
                #main window
                self.fixd.move(widget, d['x'], d['y'])
            elif offset < 353:
                #options window
                self.options.fixd.move(widget, d['x'], d['y'])
            elif offset < 396:
                #message window
                self.message.fixd.move(widget, d['x'], d['y'])
            else:
                #screen saver window
                self.scrsaver.fixd.move(widget, d['x'], d['y'])
        #other stuff
        self.options.lblHeading.set_text('Options')
        self.options.lblSettingHeading.set_text('Current Setting:')
        self.options.lblSettingValue.set_text('')

    def pop_games_list(self):
        """populate games list"""
        #which type of list is it?
        if self.current_list_idx == 0 or self.current_list_ini.get('list_type') == 'normal':
            #normal, so sort list
            list_filename = os.path.join(
                self.config_dir,
                'files',
                '%s-%s.lst' % (self.current_emu, self.current_list_idx))
            if os.path.exists(list_filename):
                self.lsGames = filters.read_filtered_list(list_filename)
            else:
                self.lsGames = []
            self.lsGames.sort()
        elif self.current_list_ini.get('list_type') in ['most_played', 'longest_played']:
            #favs type, so choose sort column
            if self.current_list_ini.get('list_type') == 'most_played':
                sort_column = FAV_TIMES_PLAYED
            else:
                sort_column = FAV_MINS_PLAYED
            #use all games to gen list
            list_filename = os.path.join(
                self.config_dir,
                'files',
                '%s-0.lst' % (self.current_emu))
            if os.path.exists(list_filename):
                self.lsGames = filters.read_filtered_list(list_filename)
            else:
                self.lsGames = []
            #create list of roms
            flist_roms = [r[GL_ROM_NAME] for r in self.lsGames]
            #order fav dictionary by number of plays
            favs = list(self.emu_favs_list.values())
            favs.sort(key = itemgetter(sort_column), reverse=True)
            #order filtered list by favs
            flist_sorted = []
            for fav in favs:
                try:
                    idx = flist_roms.index(fav[FAV_ROM_NAME])
                    flist_sorted.append(self.lsGames[idx])
                except ValueError:
                    print "%s not in list" % (fav[FAV_ROM_NAME])
            self.lsGames = flist_sorted
        #setup scroll list
        if self.current_list_idx == 0:
            self.sclGames.ls = [l[0] for l in self.lsGames]
        else:
            self.sclGames.ls = [l[0].split('(')[0].strip() for l in self.lsGames]
        #select game in list
        self.sclGames.set_selected(self.current_game_idx)
        self.on_sclGames_changed()

    def select_random_game(self):
        """randomise currently selected game number"""
        return random.randint(0, len(self.lsGames) - 1)

    def check_video_settings(self):
        """enable video playing?"""
        self.video_enabled = True
        #sorry, the mplayer control class (mplayer.py) won't work in windows
        if sys.platform == 'win32':
            self.video_enabled = False
        #check video path exists
        if not os.path.exists(self.emu_ini.get('movie_preview_path')):
            self.video_enabled = False
        #check movie artwork
        if not(self.emu_ini.getint('movie_artwork_no') > 0):
            self.video_enabled = False

    def start_timer(self, timer_type):
        """start given timer"""
        #screen saver
        if timer_type == 'scrsave':
            self.scrsave_timer = gobject.timeout_add(2500, self.on_scrsave_timer)
        #video
        elif timer_type == 'video' and self.video_enabled:
            #stop any playing vids first
            if self.video_playing:
                self.video_playing = False
                self.mplayer.close()
                self.drwVideo.set_property('visible', False)
            #restart timer
            if self.video_timer:
                gobject.source_remove(self.video_timer)
            self.video_timer = gobject.timeout_add(
                self.wahcade_ini.getint('delay_before_movie_preview') * 1000,
                self.on_video_timer)
