# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        win_scrsaver.py
# Description: screen saver window
# Copyright (c) 2006   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import os
import sys
import string

#gtk
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import pango

#project modules
from constants import *
from wc_common import WahCade
import filters

class WinScrSaver(WahCade):
    """Wah!Cade Screen Saver Window"""
    def __init__(self, WinMain):
        #set main window
        self.WinMain = WinMain
        #build the window
        self.winScrSaver = gtk.Window()
        self.fixd = gtk.Fixed()
        self.imgArtwork1 = gtk.Image()
        self.imgArtwork2 = gtk.Image()
        self.imgArtwork3 = gtk.Image()
        self.imgArtwork4 = gtk.Image()
        self.imgArtwork5 = gtk.Image()
        self.imgArtwork6 = gtk.Image()
        self.imgArtwork7 = gtk.Image()
        self.imgArtwork8 = gtk.Image()
        self.imgArtwork9 = gtk.Image()
        self.imgArtwork10 = gtk.Image()
        self.lblGameDescription = gtk.Label()
        self.lblMP3Name = gtk.Label()
        self.fixd.show()
        self.winScrSaver.add(self.fixd)
        self.lblGameDescription.show()
        self.lblMP3Name.show()
        self._main_images = [
            self.imgArtwork1,
            self.imgArtwork2,
            self.imgArtwork3,
            self.imgArtwork4,
            self.imgArtwork5,
            self.imgArtwork6,
            self.imgArtwork7,
            self.imgArtwork8,
            self.imgArtwork9,
            self.imgArtwork10]
        for widget in self._main_images:
            self.fixd.add(widget)
        self.fixd.add(self.lblGameDescription)
        self.fixd.add(self.lblMP3Name)
        self.winScrSaver.set_decorated(False)
        self.winScrSaver.set_transient_for(self.WinMain.winMain)
        self.winScrSaver.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.winScrSaver.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
        #get keyboard & mouse events
        self.winScrSaver.add_events(gtk.gdk.POINTER_MOTION_MASK | gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK)
        self.winScrSaver.connect("key_press_event", self.WinMain.on_winMain_key_press)
        self.winScrSaver.connect("key_release_event", self.WinMain.on_winMain_key_press)
        self.winScrSaver.connect("motion_notify_event", self.WinMain.on_winMain_key_press)
        self.winScrSaver.show()
        #hide mouse?
        if not self.WinMain.wahcade_ini.getint('show_cursor'):
            self.hide_mouse_cursor(self.winScrSaver)
        #interval
        self.slide_duration = 10
        self.running = False
        #fullscreen?
        if self.WinMain.config_opts.fullscreen:
            self.winScrSaver.fullscreen()

    def start_scrsaver(self, saver_type):
        """start screen saver"""
        #start duration timer
        self.video_artwork_widget = self.WinMain.video_artwork_widget
        self.slide_duration = self.WinMain.wahcade_ini.getint('slide_duration') * 1000
        self.running = True
        self.winScrSaver.show()
        if saver_type == 'slideshow':
            self.do_events()
            self.timer_id = gobject.timeout_add(self.slide_duration, self.on_slide_timer)
            #show first game
            self.on_slide_timer()

    def on_slide_timer(self):
        """change the image(s)"""
        if self.running:
            #change the images
            game_idx = self.WinMain.select_random_game()
            game_info = filters.get_game_dict(self.WinMain.lsGames[game_idx])
            #change images
            for i, img in enumerate(self._main_images):
                img_path = self.WinMain.emu_ini.get('artwork_%s_image_path' % (i + 1))
                if img_path and img.get_property('visible'):
                    #get image filename
                    img_filename = self.get_game_image(img_path, game_info)
                    #display image
                    self.display_scaled_image(img, img_filename, self.WinMain.keep_aspect)
            #set desc
            self.lblGameDescription.set_text(self.WinMain.lsGames[game_idx][GL_GAME_NAME])
        else:
            #stop, so hide window
            self.winScrSaver.hide()
        #done
        return self.running
